unit Main;
(*******************************************************************************
*                       Universal RasterFont Printer                           *
*                 RasterFontEditor component demo program                      *
*                                  by                                          *
*                            Vaclav Kourda                                     *
*                                                                              *
*                                 1998                                         *
*******************************************************************************)
interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, Menus, ComCtrls, RasterFontEditor;

type

  TConfigFileType = record
                      Width, Height : integer;

                      BitMapFont : TBitMapFont;
                      SpaceChars, SpaceRows : integer;

                      PrintX : real;
                      PrintY : real;
                      StartPrintX, StartPrintY : integer;

                      TextArray : TTextArray;
                    end;

  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    FileNewItem: TMenuItem;
    FileOpenItem: TMenuItem;
    FileSaveItem: TMenuItem;
    FilePrintItem: TMenuItem;
    FilePrintSetupItem: TMenuItem;
    FileExitItem: TMenuItem;
    HelpAboutItem: TMenuItem;
    StatusLine: TStatusBar;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    PrintDialog: TPrintDialog;
    PrintSetupDialog: TPrinterSetupDialog;
    SpeedBar: TPanel;
    SpeedButton1: TSpeedButton;  { &New }
    SpeedButton2: TSpeedButton;  { &Open... }
    SpeedButton3: TSpeedButton;  { &Save }
    SpeedButton4: TSpeedButton;  { &Print... }
    SpeedButton5: TSpeedButton;
    EditTextItem: TMenuItem;
    EditPositionItem: TMenuItem;
    N2: TMenuItem;
    SetOptions1: TMenuItem;  { E&xit }
    procedure FormCreate(Sender: TObject);
    procedure ShowHint(Sender: TObject);
    procedure FileNew(Sender: TObject);
    procedure FileOpen(Sender: TObject);
    procedure FileSave(Sender: TObject);
    procedure FilePrint(Sender: TObject);
    procedure FilePrintSetup(Sender: TObject);
    procedure FileExit(Sender: TObject);
    procedure HelpAbout(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure EditTextItemClick(Sender: TObject);
    procedure EditPositionItemClick(Sender: TObject);
    procedure SetOptions1Click(Sender: TObject);
  private
    procedure SetDefaultValues;
    function LoadCfgFile(const FileName : string) : boolean;
    function SaveCfgFile(const FileName : string) : boolean;
  public
    bmBitMap : TBitMap;
    Cfg : TConfigFileType;
    bCalculatedBitMap : boolean;
  end;

var
  MainForm: TMainForm;

implementation

uses
  TextDlg, PosDlg, Option, Printers, About;

{$R *.DFM}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Application.OnHint := ShowHint;
  FileNew(Sender);

  bmBitMap := TBitMap.Create;
  bCalculatedBitMap:=False;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  bmBitMap.Free;
end;

function TMainForm.LoadCfgFile(const FileName : string) : boolean;
var F : File;
    i,j : integer;
begin
  if FileExists(FileName) then
  begin
    AssignFile(F,FileName);
    try
      Reset(F,1);
      BlockRead(F,Cfg.Width,SizeOf(Cfg.Width));
      BlockRead(F,Cfg.Height,SizeOf(Cfg.Height));
      BlockRead(F,Cfg.BitMapFont,SizeOf(Cfg.BitMapFont));
      BlockRead(F,Cfg.SpaceChars,SizeOf(Cfg.SpaceChars));
      BlockRead(F,Cfg.SpaceRows,SizeOf(Cfg.SpaceRows));
      BlockRead(F,Cfg.PrintX,SizeOf(Cfg.PrintX));
      BlockRead(F,Cfg.PrintY,SizeOf(Cfg.PrintY));
      BlockRead(F,Cfg.StartPrintX,SizeOf(Cfg.StartPrintX));
      BlockRead(F,Cfg.StartPrintY,SizeOf(Cfg.StartPrintY));
      for i:=1 to Cfg.Height do
        for j:=1 to Cfg.Width do
          begin
          BlockRead(F,Cfg.TextArray[i,j],sizeof(TCharItem));
          end;
      Result := True;
    finally
      CloseFile(F);
    end;
  end
  else
    Result := False;
end;

function TMainForm.SaveCfgFile(const FileName : string) : boolean;
var F : File;
    i,j : integer;
begin
  AssignFile(F,FileName);
  try
    ReWrite(F,1);
    BlockWrite(F,Cfg.Width,SizeOf(Cfg.Width));
    BlockWrite(F,Cfg.Height,SizeOf(Cfg.Height));
    BlockWrite(F,Cfg.BitMapFont,SizeOf(Cfg.BitMapFont));
    BlockWrite(F,Cfg.SpaceChars,SizeOf(Cfg.SpaceChars));
    BlockWrite(F,Cfg.SpaceRows,SizeOf(Cfg.SpaceRows));
    BlockWrite(F,Cfg.PrintX,SizeOf(Cfg.PrintX));
    BlockWrite(F,Cfg.PrintY,SizeOf(Cfg.PrintY));
    BlockWrite(F,Cfg.StartPrintX,SizeOf(Cfg.StartPrintX));
    BlockWrite(F,Cfg.StartPrintY,SizeOf(Cfg.StartPrintY));
    for i:=1 to Cfg.Height do
      for j:=1 to Cfg.Width do
        BlockWrite(F,Cfg.TextArray[i,j],sizeof(TCharItem));
    Result := True;
  finally
    CloseFile(F);
  end;
end;

procedure TMainForm.ShowHint(Sender: TObject);
begin
  StatusLine.SimpleText := Application.Hint;
end;

procedure TMainForm.FileNew(Sender: TObject);
var i,j : integer;
begin
  Cfg.Width := MAX_SIZE;
  Cfg.Height := MAX_SIZE;

  Cfg.BitMapFont.FontFile :='Courier.fnt';
  Cfg.BitMapFont.FontHeight := 47;
  Cfg.BitMapFont.FontWidth := 23;
  Cfg.BitMapFont.LineSize := 1;
  Cfg.BitMapFont.CharSetWidth := 20;
  Cfg.BitMapFont.CharSetHeight := 13;

  Cfg.SpaceChars := 1;
  Cfg.SpaceRows := 1;
  Cfg.PrintX := 1;
  Cfg.PrintY := 1;
  Cfg.StartPrintX := 100;
  Cfg.StartPrintY := 100;

  for i:=1 to MAX_SIZE do
    for j:=1 to MAX_SIZE do
    begin
      Cfg.TextArray[i,j].Top := 0;
      Cfg.TextArray[i,j].Left := 0;
      Cfg.TextArray[i,j].Width := 0;
      Cfg.TextArray[i,j].Height := 0;
      Cfg.TextArray[i,j].Color := clBlack;
      Cfg.TextArray[i,j].Character := ' ';
    end;

  SetDefaultValues;
end;

procedure TMainForm.FileOpen(Sender: TObject);
begin
  if OpenDialog.Execute then
  begin
    LoadCfgFile(OpenDialog.FileName);
    bCalculatedBitMap := False;
  end;
end;

procedure TMainForm.FileSave(Sender: TObject);
begin
  if SaveDialog.Execute then
  begin
    SaveCfgFile(SaveDialog.FileName);
  end;
end;

procedure TMainForm.FilePrintSetup(Sender: TObject);
begin
  PrintSetupDialog.Execute;
end;

procedure TMainForm.FileExit(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.HelpAbout(Sender: TObject);
begin
  AboutBox.ShowModal;
end;

procedure TMainForm.EditTextItemClick(Sender: TObject);
var i,j : integer;
    sTemp : string;
begin
  EditTextDialog.Memo.Clear;
  for i:=1 to Cfg.Height do
  begin
    sTemp := '';
    for j:=1 to Cfg.Width do
    begin
      sTemp := sTemp+Cfg.TextArray[i,j].Character;
    end;
    EditTextDialog.Memo.Lines.Add(sTemp);
  end;

  if EditTextDialog.ShowModal=mrOk then
  begin
    for i:=1 to Cfg.Height do
    begin
      sTemp := EditTextDialog.Memo.Lines.Strings[i-1];
      for j:=1 to Cfg.Width do
      begin
        if Length(sTemp)<j then
          Cfg.TextArray[i,j].Character := ' '
        else
          Cfg.TextArray[i,j].Character := sTemp[j];
      end;
    end;
    bCalculatedBitMap := False;
  end;
end;

procedure TMainForm.SetDefaultValues;
var i,j : integer;
    xPos, yPos : integer;
begin
  yPos := 0;
  for i:=1 to Cfg.Height do
  begin
    xPos := 0;
    for j:=1 to Cfg.Width do
    begin
      Cfg.TextArray[i,j].Width := Cfg.BitMapFont.FontWidth;
      Cfg.TextArray[i,j].Height := Cfg.BitMapFont.FontHeight;
      Cfg.TextArray[i,j].Top := yPos;
      Cfg.TextArray[i,j].Left := xPos;
      Inc(xPos,Cfg.BitMapFont.FontWidth+Cfg.SpaceChars);
    end;
      Inc(yPos,Cfg.BitMapFont.FontHeight+Cfg.SpaceRows)
  end;
end;

procedure TMainForm.EditPositionItemClick(Sender: TObject);
var i, j : integer;
begin
  Screen.Cursor := crHourGlass;
  try
    with SetPosDialog.rfEdit do
    begin
      Clear;
      TextArrayHeight:=Cfg.Height;
      TextArrayWidth:=Cfg.Width;
      SpaceChars:=Cfg.SpaceChars;
      SpaceRows:=Cfg.SpaceRows;
      BitMapFont := Cfg.BitMapFont;
      for i:=1 to TextArrayHeight do
        for j:=1 to TextArrayWidth do
          begin
            SetArrayItem(i,j,Cfg.TextArray[i,j],False);
          end;
      DrawTextArray;
      SpaceSelect := False;
      FixHorPos := SetPosDialog.cbFixHor.Checked;
      FixVerPos := SetPosDialog.cbFixVer.Checked;
      FixWidth := SetPosDialog.cbFixWidth.Checked;
      FixHeight := SetPosDialog.cbFixHeight.Checked;
    end;
  finally
    Screen.Cursor := crDefault;
  end;

  if SetPosDialog.ShowModal=mrOk then
  begin
    for i:=1 to SetPosDialog.rfEdit.TextArrayHeight do
      for j:=1 to SetPosDialog.rfEdit.TextArrayWidth do
        begin
          Cfg.TextArray[i,j]:=SetPosDialog.rfEdit.GetArrayItem(i,j);
        end;
    bmBitMap.Width :=SetPosDialog.rfEdit.bmPicture.Width;
    bmBitMap.Height :=SetPosDialog.rfEdit.bmPicture.Height;
    bmBitMap.Canvas.Draw(0,0,SetPosDialog.rfEdit.bmPicture);
    bCalculatedBitMap := True;
  end
end;


procedure TMainForm.SetOptions1Click(Sender: TObject);
begin
  OptionDlg.Conf := Cfg;
  if OptionDlg.ShowModal=mrOk then
  begin
    Cfg:=OptionDlg.Conf;
    bCalculatedBitMap := False;
  end;
end;

procedure TMainForm.FilePrint(Sender: TObject);
var i,j : integer;
begin
  if PrintDialog.Execute then
  begin
    if not bCalculatedBitMap then
    begin
      Screen.Cursor := crHourGlass;
      try
        with SetPosDialog.rfEdit do
        begin
          Clear;
          TextArrayHeight:=Cfg.Height;
          TextArrayWidth:=Cfg.Width;
          SpaceChars:=Cfg.SpaceChars;
          SpaceRows:=Cfg.SpaceRows;
          BitMapFont := Cfg.BitMapFont;
          for i:=1 to TextArrayHeight do
            for j:=1 to TextArrayWidth do
              begin
                SetArrayItem(i,j,Cfg.TextArray[i,j],False);
              end;
          DrawTextArray;
        end;
        bmBitMap.Width :=SetPosDialog.rfEdit.bmPicture.Width;
        bmBitMap.Height :=SetPosDialog.rfEdit.bmPicture.Height;
        bmBitMap.Canvas.Draw(0,0,SetPosDialog.rfEdit.bmPicture);
        bCalculatedBitMap := True;
      finally
        Screen.Cursor := crDefault;
      end;
    end;

    if PrintDialog.PrintToFile then
    begin
      if SaveDialog.Execute then
      begin
        bmBitMap.SaveToFile(SaveDialog.FileName);
      end;
    end
    else
    begin
    Printer.BeginDoc;
    try
      Printer.Canvas.StretchDraw(Rect(Cfg.StartPrintX,Cfg.StartPrintY,
                                      Round(Cfg.StartPrintX+bmBitMap.Width*Cfg.PrintX),
                                      Round(Cfg.StartPrintY+bmBitMap.Height*Cfg.PrintY)),
                                      bmBitMap);
    finally
      Printer.EndDoc;
    end;
    end;
  end;
end;

end.
