unit Option;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, Dialogs, RasterFontEditor, Main;

type
  TOptionDlg = class(TForm)
    Panel1: TPanel;
    OKBtn: TButton;
    CancelBtn: TButton;
    gbFont: TGroupBox;
    Label1: TLabel;
    btnFont: TSpeedButton;
    Label2: TLabel;
    Label3: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label4: TLabel;
    eFontFile: TEdit;
    eCHeight: TEdit;
    eCWidth: TEdit;
    eCSWidth: TEdit;
    eLSize: TEdit;
    eCSHeight: TEdit;
    gbPrint: TGroupBox;
    Label7: TLabel;
    Label8: TLabel;
    eStretchX: TEdit;
    eStretchY: TEdit;
    Label12: TLabel;
    eSChars: TEdit;
    Label13: TLabel;
    eSRows: TEdit;
    Label11: TLabel;
    eTHeight: TEdit;
    Label10: TLabel;
    eTWidth: TEdit;
    ePPX: TEdit;
    Label9: TLabel;
    Label14: TLabel;
    ePPY: TEdit;
    OpenFontDialog: TOpenDialog;
    procedure btnFontClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    Conf : TConfigFileType;
    function ValidItems : boolean;
  end;

var
  OptionDlg: TOptionDlg;

implementation

{$R *.DFM}

procedure TOptionDlg.btnFontClick(Sender: TObject);
begin
  OpenFontDialog.FileName := Conf.BitMapFont.FontFile;
  if OpenFontDialog.Execute then
  begin
    Conf.BitMapFont.FontFile := OpenFontDialog.FileName;
    eFontFile.Text := OpenFontDialog.FileName;
  end;
end;

procedure TOptionDlg.FormShow(Sender: TObject);
begin
  eFontFile.Text := Conf.BitMapFont.FontFile;
  eCHeight.Text := IntToStr(Conf.BitMapFont.FontHeight);
  eCWidth.Text := IntToStr(Conf.BitMapFont.FontWidth);
  eCSHeight.Text := IntToStr(Conf.BitMapFont.CharSetHeight);
  eCSWidth.Text := IntToStr(Conf.BitMapFont.CharSetWidth);
  eLSize.Text := IntToStr(Conf.BitMapFont.LineSize);

  eTHeight.Text := IntToStr(Conf.Height);
  eTWidth.Text := IntToStr(Conf.Width);
  eSChars.Text := IntToStr(Conf.SpaceChars);
  eSRows.Text := IntToStr(Conf.SpaceRows);

  eStretchX.Text := FloatToStr(Conf.PrintX);
  eStretchY.Text := FloatToStr(Conf.PrintY);
  ePPX.Text := IntToStr(Conf.StartPrintX);
  ePPY.Text := IntToStr(Conf.StartPrintY);
end;

procedure TOptionDlg.OKBtnClick(Sender: TObject);
begin
  if ValidItems then
    begin
      ModalResult := mrOk;
    end;
end;

function TOptionDlg.ValidItems : boolean;
var iTemp : integer;
    eTemp : Extended;
begin
  Result := True;
  if FileExists(eFontFile.Text) then
    Conf.BitMapFont.FontFile := eFontFile.Text
  else
  begin
    Result := False;
    MessageDlg('Invalid fontfile name!',mtError,[mbOk],0);
    ActiveControl:=eFontFile;
  end;

  if Result then
  begin
    try
      iTemp := StrToInt(eCHeight.Text);
      if (iTemp<=0) or (iTemp>=1000) then
        raise EConvertError.Create('');
      Conf.BitMapFont.FontHeight := iTemp;
    except
      on E : Exception do
      begin
        Result := False;
        MessageDlg('Invalid character height!',mtError,[mbOk],0);
        ActiveControl:=eCHeight;
      end;
    end;
  end;

  if Result then
  begin
    try
      iTemp := StrToInt(eCWidth.Text);
      if (iTemp<=0) or (iTemp>=1000) then
        raise EConvertError.Create('');
      Conf.BitMapFont.FontWidth := iTemp;
    except
      on E : Exception do
      begin
        Result := False;
        MessageDlg('Invalid character width!',mtError,[mbOk],0);
        ActiveControl:=eCWidth;
      end;
    end;
  end;

  if Result then
  begin
    try
      iTemp := StrToInt(eCSHeight.Text);
      if (iTemp<=0) or (iTemp>=1000) then
        raise EConvertError.Create('');
      Conf.BitMapFont.CharSetHeight := iTemp;
    except
      on E : Exception do
      begin
        Result := False;
        MessageDlg('Invalid characterset height!',mtError,[mbOk],0);
        ActiveControl:=eCSHeight;
      end;
    end;
  end;

  if Result then
  begin
    try
      iTemp := StrToInt(eCSWidth.Text);
      if (iTemp<=0) or (iTemp>=1000) then
        raise EConvertError.Create('');
      Conf.BitMapFont.CharSetWidth := iTemp;
    except
      on E : Exception do
      begin
        Result := False;
        MessageDlg('Invalid characterset width!',mtError,[mbOk],0);
        ActiveControl:=eCSWidth;
      end;
    end;
  end;

  if Result then
  begin
    try
      iTemp := StrToInt(eLSize.Text);
      if (iTemp<0) or (iTemp>1000) then
        raise EConvertError.Create('');
      Conf.BitMapFont.LineSize := iTemp;
    except
      on E : Exception do
      begin
        Result := False;
        MessageDlg('Invalid line size!',mtError,[mbOk],0);
        ActiveControl:=eLSize;
      end;
    end;
  end;

  if Result then
  begin
    try
      iTemp := StrToInt(eTHeight.Text);
      if (iTemp<=0) or (iTemp>=1000) then
        raise EConvertError.Create('');
      Conf.Height := iTemp;
    except
      on E : Exception do
      begin
        Result := False;
        MessageDlg('Invalid text height!',mtError,[mbOk],0);
        ActiveControl:=eTHeight;
      end;
    end;
  end;

  if Result then
  begin
    try
      iTemp := StrToInt(eTWidth.Text);
      if (iTemp<=0) or (iTemp>1000) then
        raise EConvertError.Create('');
      Conf.Width := iTemp;
    except
      on E : Exception do
      begin
        Result := False;
        MessageDlg('Invalid text width!',mtError,[mbOk],0);
        ActiveControl:=eTWidth;
      end;
    end;
  end;

  if Result then
  begin
    try
      iTemp := StrToInt(eSChars.Text);
      if (iTemp<0) or (iTemp>1000) then
        raise EConvertError.Create('');
      Conf.SpaceChars := iTemp;
    except
      on E : Exception do
      begin
        Result := False;
        MessageDlg('Invalid space size between characters!',mtError,[mbOk],0);
        ActiveControl:=eSChars;
      end;
    end;
  end;

  if Result then
  begin
    try
      iTemp := StrToInt(eSRows.Text);
      if (iTemp<0) or (iTemp>1000) then
        raise EConvertError.Create('');
      Conf.SpaceRows := iTemp;
    except
      on E : Exception do
      begin
        Result := False;
        MessageDlg('Invalid space size between rows!',mtError,[mbOk],0);
        ActiveControl:=eSRows;
      end;
    end;
  end;

  if Result then
  begin
    try
      eTemp := StrToFloat(eStretchX.Text);
      if (eTemp<0) or (eTemp>10) then
        raise EConvertError.Create('');
      Conf.PrintX := eTemp;
    except
      on E : Exception do
      begin
        Result := False;
        MessageDlg('Invalid stretching number!',mtError,[mbOk],0);
        ActiveControl:=eStretchX;
      end;
    end;
  end;

  if Result then
  begin
    try
      eTemp := StrToFloat(eStretchY.Text);
      if (eTemp<0) or (eTemp>10) then
        raise EConvertError.Create('');
      Conf.PrintY := eTemp;
    except
      on E : Exception do
      begin
        Result := False;
        MessageDlg('Invalid stretching number!',mtError,[mbOk],0);
        ActiveControl:=eStretchY;
      end;
    end;
  end;

  if Result then
  begin
    try
      iTemp := StrToInt(ePPX.Text);
      if (iTemp<0) or (iTemp>10000) then
        raise EConvertError.Create('');
      Conf.StartPrintX := iTemp;
    except
      on E : Exception do
      begin
        Result := False;
        MessageDlg('Invalid printing start x position!',mtError,[mbOk],0);
        ActiveControl:=ePPX;
      end;
    end;
  end;

  if Result then
  begin
    try
      iTemp := StrToInt(ePPY.Text);
      if (iTemp<0) or (iTemp>10000) then
        raise EConvertError.Create('');
      Conf.StartPrintY := iTemp;
    except
      on E : Exception do
      begin
        Result := False;
        MessageDlg('Invalid printing start y position!',mtError,[mbOk],0);
        ActiveControl:=ePPY;
      end;
    end;
  end;
end;

end.
